/*
** packf.c -- pack a nmh folder into a file
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <h/mh.h>
#include <fcntl.h>
#include <h/dropsbr.h>
#include <h/utils.h>
#include <errno.h>

static struct swit switches[] = {
#define VERSIONSW  0
	{ "version", 0 },
#define HELPSW  1
	{ "help", 0 },
	{ NULL, 0 }
};


int
main(int argc, char **argv)
{
	int fd, msgnum;
	char *cp, *maildir, *msgnam, *folder = NULL, buf[BUFSIZ];
	char **argp, **arguments;
	struct msgs_array msgs = { 0, 0, NULL };
	struct msgs *mp;

#ifdef LOCALE
	setlocale(LC_ALL, "");
#endif
	invo_name = mhbasename(argv[0]);

	/* read user profile/context */
	context_read();

	arguments = getarguments(invo_name, argc, argv, 1);
	argp = arguments;

	/*
	** Parse arguments
	*/
	while ((cp = *argp++)) {
		if (*cp == '-') {
			switch (smatch(++cp, switches)) {
			case AMBIGSW:
				ambigsw(cp, switches);
				done(1);
			case UNKWNSW:
				adios(NULL, "-%s unknown", cp);

			case HELPSW:
				snprintf(buf, sizeof(buf), "%s [+folder] [msgs] [switches]", invo_name);
				print_help(buf, switches, 1);
				done(1);
			case VERSIONSW:
				print_version(invo_name);
				done(1);
			}
		}
		if (*cp == '+' || *cp == '@') {
			if (folder)
				adios(NULL, "only one folder at a time!");
			folder = getcpy(expandfol(cp));
		} else
			app_msgarg(&msgs, cp);
	}

	/* default is to pack whole folder */
	if (!msgs.size)
		app_msgarg(&msgs, seq_all);

	if (!folder)
		folder = getcurfol();
	maildir = toabsdir(folder);

	if (chdir(maildir) == NOTOK)
		adios(maildir, "unable to change directory to ");

	/* read folder and create message structure */
	if (!(mp = folder_read(folder)))
		adios(NULL, "unable to read folder %s", folder);

	/* check for empty folder */
	if (mp->nummsg == 0)
		adios(NULL, "no messages in %s", folder);

	/* parse all the message ranges/sequences and set SELECTED */
	for (msgnum = 0; msgnum < msgs.size; msgnum++)
		if (!m_convert(mp, msgs.msgs[msgnum]))
			done(1);
	seq_setprev(mp);  /* set the previous-sequence */

	/* copy all the SELECTED messages to stdout */
	for (msgnum = mp->lowsel; msgnum <= mp->hghsel; msgnum++) {
		if (is_selected(mp, msgnum)) {
			if ((fd = open(msgnam = m_name(msgnum), O_RDONLY))
					== NOTOK) {
				admonish(msgnam, "unable to read message");
				break;
			}
			if (mbox_copy(fileno(stdout), fd) == NOTOK) {
				adios(NULL, "error writing to stdout");
			}
			close(fd);
		}
	}
	context_replace(curfolder, folder);
	if (mp->hghsel != mp->curmsg)
		seq_setcur(mp, mp->lowsel);
	seq_save(mp);
	context_save();
	folder_free(mp);
	done(0);
	return 1;
}
